use crate::actors::vnc_tokens_actor::VNC_TOKEN_LIFETIME;
use crate::app_config::AppConfig;
use crate::constants;
use crate::constants::{DISK_NAME_MAX_LEN, DISK_NAME_MIN_LEN, DISK_SIZE_MAX, DISK_SIZE_MIN};
use crate::controllers::{HttpResult, LibVirtReq};
use crate::extractors::local_auth_extractor::LocalAuthEnabled;
use crate::libvirt_rest_structures::hypervisor::HypervisorInfo;
use actix_web::{HttpResponse, Responder};
use sysinfo::{NetworksExt, System, SystemExt};

#[derive(serde::Serialize)]
struct StaticConfig {
    auth_disabled: bool,
    local_auth_enabled: bool,
    oidc_auth_enabled: bool,
    iso_mimetypes: &'static [&'static str],
    net_mac_prefix: &'static str,
    constraints: ServerConstraints,
    builtin_nwfilter_rules: &'static [&'static str],
}

#[derive(serde::Serialize)]
struct LenConstraints {
    min: usize,
    max: usize,
}

#[derive(serde::Serialize)]
struct ServerConstraints {
    iso_max_size: usize,
    vnc_token_duration: u64,
    vm_name_size: LenConstraints,
    vm_title_size: LenConstraints,
    memory_size: LenConstraints,
    disk_name_size: LenConstraints,
    disk_size: LenConstraints,
    net_name_size: LenConstraints,
    net_title_size: LenConstraints,
    dhcp_reservation_host_name: LenConstraints,
}

pub async fn static_config(local_auth: LocalAuthEnabled) -> impl Responder {
    HttpResponse::Ok().json(StaticConfig {
        auth_disabled: AppConfig::get().unsecure_disable_auth,
        local_auth_enabled: *local_auth,
        oidc_auth_enabled: !AppConfig::get().disable_oidc,
        iso_mimetypes: &constants::ALLOWED_ISO_MIME_TYPES,
        net_mac_prefix: constants::NET_MAC_ADDR_PREFIX,
        builtin_nwfilter_rules: &constants::BUILTIN_NETWORK_FILTER_RULES,
        constraints: ServerConstraints {
            iso_max_size: constants::ISO_MAX_SIZE,

            vnc_token_duration: VNC_TOKEN_LIFETIME,

            vm_name_size: LenConstraints { min: 2, max: 50 },
            vm_title_size: LenConstraints { min: 0, max: 50 },
            memory_size: LenConstraints {
                min: constants::MIN_VM_MEMORY,
                max: constants::MAX_VM_MEMORY,
            },
            disk_name_size: LenConstraints {
                min: DISK_NAME_MIN_LEN,
                max: DISK_NAME_MAX_LEN,
            },
            disk_size: LenConstraints {
                min: DISK_SIZE_MIN,
                max: DISK_SIZE_MAX,
            },

            net_name_size: LenConstraints { min: 2, max: 50 },
            net_title_size: LenConstraints { min: 0, max: 50 },

            dhcp_reservation_host_name: LenConstraints { min: 2, max: 250 },
        },
    })
}

#[derive(serde::Serialize)]
struct ServerInfo {
    hypervisor: HypervisorInfo,
    system: System,
}

pub async fn server_info(client: LibVirtReq) -> HttpResult {
    let mut system = System::new();
    system.refresh_disks_list();
    system.refresh_components_list();
    system.refresh_networks_list();
    system.refresh_all();

    Ok(HttpResponse::Ok().json(ServerInfo {
        hypervisor: client.get_info().await?,
        system,
    }))
}

pub async fn number_vcpus() -> HttpResult {
    let mut system = System::new();
    system.refresh_cpu();
    let number_cpus = system.cpus().len();
    assert_ne!(number_cpus, 0, "Got invlid number of CPU!");

    let mut possible_numbers = vec![1];

    if number_cpus > 1 {
        for i in 0..(number_cpus / 2) {
            possible_numbers.push(2 + 2 * i);
        }
    }

    Ok(HttpResponse::Ok().json(possible_numbers))
}

pub async fn networks_list() -> HttpResult {
    let mut system = System::new();
    system.refresh_networks_list();

    Ok(HttpResponse::Ok().json(
        system
            .networks()
            .iter()
            .map(|n| n.0.to_string())
            .collect::<Vec<_>>(),
    ))
}
